﻿using System;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Ink;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using System.Xml.Linq;

namespace xamPivotGrid2
{
    public interface IXmlDataProvider
    {
        /// <summary>
        /// Occurs when finished loading a xml file.
        /// </summary>
        event EventHandler<GetXmlDataCompletedEventArgs> GetXmlDataCompleted;

        void GetXmlDataAsync(string xmlFileName);

    }
    public class GetXmlDataCompletedEventArgs : EventArgs
    {

        public GetXmlDataCompletedEventArgs(XDocument xDocument)
        {
            this.Result = xDocument;
            this.Error = null;
            this.HasError = false;
            this.UserState = null;
        }
        public GetXmlDataCompletedEventArgs(XDocument xDocument, object userState)
        {
            this.Result = xDocument;
            this.Error = null;
            this.HasError = false;
            this.UserState = userState;
        }
        public GetXmlDataCompletedEventArgs(Exception error)
        {
            this.Result = null;
            this.Error = error;
            this.HasError = true;
            this.UserState = null;
        }

        /// <summary>
        /// Gets the loaded xml file as a XDocument object
        /// </summary>
        public XDocument Result { get; private set; }
        /// <summary>
        /// Gets an Exception assosicated with loading the xml file
        /// </summary>
        public Exception Error { get; private set; }
        /// <summary>
        /// Gets if there were error when loading the xml file
        /// </summary>
        public bool HasError { get; private set; }
        public object UserState { get; set; }

    }
}
