﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using System.ComponentModel;
using Infragistics.Olap.FlatData;
using System.Collections;
using Infragistics.Olap;
using System.Threading;

namespace xamPivotGrid2
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        private BackgroundWorker loaderbw;

        public MainWindow()
        {
            InitializeComponent();
            InitializeLoaderBw();
        }

        private void Window_Loaded(object sender, RoutedEventArgs e)
        {
            //Generate data
            loaderbw.RunWorkerAsync("Generating and Loading Data");

            int numberOfSalesToGenerate = 10000;

            BackgroundWorker bw = new BackgroundWorker();
            bw.DoWork += (sender2, e2) =>
            {
                e2.Result = SalesDataGenerator.GenerateSales(numberOfSalesToGenerate);
            };
            bw.RunWorkerCompleted += (sender2, e2) =>
            {
                IEnumerable GeneratedData = e2.Result as IEnumerable;

                //Create a DataSource from that data
                FlatDataSource dataSource = new FlatDataSource()
                {
                    Cube = DataSourceBase.GenerateInitialCube("Sale"),
                    Rows = DataSourceBase.GenerateInitialItems("[City].[City]"),
                    Columns = DataSourceBase.GenerateInitialItems("[Product].[Product]"),
                    Measures = DataSourceBase.GenerateInitialItems("AmountOfSale"),
                    ItemsSource = GeneratedData
                };

                dataSource.PropertyChanged += DataSource_PropertyChanged;

                dataSource.CubesSettings.Add(GenerateCubeMetadata());
                dataSource.AddHierarchyDescriptor(GenerateSellerHierarchy());
                dataSource.AddHierarchyDescriptor(GenerateProductHierarchy());
                dataSource.AddHierarchyDescriptor(GenerateLocationHierarchy());

                //Assign the DataSource to the xamPivotGrid and xamPivotDataSelector
                BackgroundWorker dataBw = new BackgroundWorker();
                dataBw.DoWork += (sender3, e3) =>
                {
                    this.pivotGrid.Dispatcher.BeginInvoke((Action)(() =>
                    {
                        this.pivotGrid.DataSource = dataSource;
                    }));
                    this.dataSelector.Dispatcher.BeginInvoke((Action)(() =>
                    {
                        this.dataSelector.DataSource = dataSource;
                    }));
                };
                dataBw.RunWorkerCompleted += (sender3, e3) =>
                {
                    loaderbw.CancelAsync();
                };
                dataBw.RunWorkerAsync();
            };
            bw.RunWorkerAsync();
        }

        void DataSource_PropertyChanged(object sender, PropertyChangedEventArgs e)
        {
            if (e.PropertyName == "Processing")
            {
                if (this.pivotGrid.DataSource.Processing)
                {

                    MessageTextBlock.Visibility = Visibility.Visible;
                }
                else
                {
                    MessageTextBlock.Visibility = Visibility.Collapsed;
                }
            }
        }

        private void InitializeLoaderBw()
        {
            loaderbw = new BackgroundWorker();
            loaderbw.WorkerSupportsCancellation = true;
            loaderbw.DoWork += (sender, e) =>
            {
                string message = e.Argument as string;
                int dots = 0;
                while (!loaderbw.CancellationPending)
                {
                    string tempMessage = message;
                    if (dots > 3) dots = 0;
                    for (int i = 0; i < dots; i++)
                    {
                        tempMessage += ".";
                    }
                    dots++;

                    if (!e.Cancel)
                    {
                        //loaderText.Dispatcher.BeginInvoke((Action)(() =>
                        //{
                        //    loaderText.Text = tempMessage;
                        //}));

                        Thread.Sleep(400);
                    }
                }
            };
        }

        private static HierarchyDescriptor<Sale> GenerateSellerHierarchy()
        {
            HierarchyDescriptor<Sale> sellerHierarchy = new HierarchyDescriptor<Sale>(p => p.Seller);
            sellerHierarchy.AddLevel(p =>
                "All sellers",
                "All sellers"
                );
            sellerHierarchy.AddLevel(p =>
                p.Seller.Name,
                "Seller name"
                );
            return sellerHierarchy;
        }

        private static HierarchyDescriptor<Sale> GenerateProductHierarchy()
        {
            HierarchyDescriptor<Sale> productHierarchy = new HierarchyDescriptor<Sale>(p => p.Product);
            productHierarchy.AddLevel(p =>
                "All products",
                "All products"
                );
            productHierarchy.AddLevel(p =>
                p.Product.Name,
                "Product name"
                );
            return productHierarchy;
        }

        private static HierarchyDescriptor<Sale> GenerateLocationHierarchy()
        {
            HierarchyDescriptor<Sale> locationHierarchy = new HierarchyDescriptor<Sale>(p => p.City);
            locationHierarchy.AddLevel(p =>
                "All locations",
                "All locations"
                );
            locationHierarchy.AddLevel(p =>
                p.City,
                "City"
                );
            return locationHierarchy;
        }

        private CubeMetadata GenerateCubeMetadata()
        {
            CubeMetadata cubeMetadata = new CubeMetadata();
            cubeMetadata.DisplayName = "Metadata 1";
            cubeMetadata.DataTypeFullName = typeof(Sale).FullName;
            cubeMetadata.DimensionSettings.Add(new DimensionMetadata()
            {
                SourcePropertyName = "AmountOfSale",
                DisplayName = "Amount Of Sale",
                DisplayFormat = "{0:C2}"
            });
            cubeMetadata.DimensionSettings.Add(new DimensionMetadata()
            {
                SourcePropertyName = "NumberOfUnits",
                DisplayName = "Number Of Units"
            });
            cubeMetadata.DimensionSettings.Add(new DimensionMetadata()
            {
                SourcePropertyName = "Product",
                DisplayName = "Product"
            });
            cubeMetadata.DimensionSettings.Add(new DimensionMetadata()
            {
                SourcePropertyName = "City",
                DisplayName = "City"
            });
            cubeMetadata.DimensionSettings.Add(new DimensionMetadata()
            {
                SourcePropertyName = "Seller",
                DisplayName = "Seller"
            });
            cubeMetadata.DimensionSettings.Add(new DimensionMetadata()
            {
                SourcePropertyName = "Date",
                DisplayName = "Date"
            });

            return cubeMetadata;
        }
    }
}
